@@ -23,7 +23,7 @@ DATA_FILE = BASE_DIR + 'Tubes.dat'
 CLOSURES_FILE = BASE_DIR + 'Closures.txt'
 CONFIG_FILE = 'Config.txt'
 
-MAP_IMAGE = os.path.dirname(argv[0]) + "/tfl_map.png"
+MAP_IMAGE = os.path.dirname(argv[0]) + "/tfl_map.gif"
 
 CLOSURES_TIMEOUT = 300
 
@@ -44,7 +44,8 @@ STATION_LINES = 3
 STATION_NOCHANGE = 4
 STATION_TIMES = 5
 STATION_DISTANCE = 6
-Station = [{}, {}, {}, {}, {}, {}, {}]
+STATION_COORDS = 7
+Station = [{}, {}, {}, {}, {}, {}, {}, {}]
 
 LINE_STOP = 0
 LINE_DIRECTIONS = 1
@@ -427,7 +428,8 @@ def guide(station_from, station_to):
 	if (time() - closures_time) > CLOSURES_TIMEOUT:
 		closures = get_closures()
 		closures_time = time()
-	return parse(plan(matchStation(station_from), matchStation(station_to)))
+	route = plan(matchStation(station_from), matchStation(station_to))
+	return [parse(route), route]
 
 
 ###############################################################################
@@ -440,7 +442,7 @@ def main():
 		from_entry.set_text(to_entry.get_text())
 		to_entry.set_text(tmp)
 
-	def togglemap(mapbuf, map_x, map_y):
+	def togglemap(mapbuf, map_x, map_y, title, route, do_route=False):
 		win = hildon.StackableWindow()
 		win.set_title("Tube map")
 		win.fullscreen()
@@ -450,13 +452,62 @@ def main():
 		image.set_from_pixbuf(mapbuf)
 		image_event = gtk.EventBox()
 		image_event.add(image)
-		image_event.connect("button_press_event", mapclick, pannable_area)
+		if route.plan != None:
+			for stop in route.plan[0]:
+				#Station[STATION_COORDS][stop][0], Station[STATION_COORDS][stop][1], Station[STATION_COORDS][stop][2], Station[STATION_COORDS][stop][3]
+				pass
+		if not do_route:
+			image_event.connect("button_press_event", mapclick, pannable_area)
+		else:
+			image_event.connect("button_press_event", lambda widget,event,data: maproute(widget,event,data,title,route), pannable_area)
 		image_event.oldtime = 0
 		image_event.container = win
+		win.start = ""
+		win.end = ""
 		pannable_area.add_with_viewport(image_event)
 		win.add(pannable_area)
 		win.show_all()
 		pannable_area.jump_to(map_x, map_y)
+		if do_route:
+			hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Choose starting point (double-click)...")
+
+	def maproute(widget, event, data, title, route):
+		if (event.time - widget.oldtime) < 250:
+			if (widget.container.start == "") or (widget.container.end == ""):
+				minimum = ["", INFINITY]
+				for station in Station[STATION_COORDS]:
+					if len(Station[STATION_COORDS][station]) == 4:
+						if (event.x >= Station[STATION_COORDS][station][0]) and (event.x <= Station[STATION_COORDS][station][2]) and (event.y >= Station[STATION_COORDS][station][1]) and (event.y <= Station[STATION_COORDS][station][3]):
+							minimum = [station, 0]
+						if minimum[1] > 0:
+							dist = ((Station[STATION_COORDS][station][0] + Station[STATION_COORDS][station][2])/2. - event.x)**2 + ((Station[STATION_COORDS][station][1] + Station[STATION_COORDS][station][3])/2. - event.y)**2
+							if dist < minimum[1]:
+								minimum = [station, dist]
+				if widget.container.start == "":
+					widget.container.start = minimum[0]
+					hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Station chosen: %s. Choose destination (double-click)..." % Station[STATION_NAME][minimum[0]])
+				else:
+					widget.container.end = minimum[0]
+					hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Station chosen: %s." % Station[STATION_NAME][minimum[0]])
+					prompt = hildon.hildon_note_new_confirmation(widget.container, "Do you want to route from %s to %s?" % (Station[STATION_NAME][widget.container.start], Station[STATION_NAME][widget.container.end]))
+					response = gtk.Dialog.run(prompt)
+					prompt.destroy()
+					if response == gtk.RESPONSE_OK:
+						title.station_from = widget.container.start
+						title.station_to = widget.container.end
+						plan = guide(title.station_from, title.station_to)
+						plan[0] = plan[0].replace('&', '&amp;')
+						title.set_markup("<big>%s to %s</big>" % (Station[STATION_NAME][matchStation(title.station_from)], Station[STATION_NAME][matchStation(title.station_to)]))
+						route.get_buffer().set_text(plan[0])
+						route.plan = plan[1]
+						hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Routing complete")
+						widget.container.destroy()
+					elif response == gtk.RESPONSE_CANCEL:
+						widget.container.destroy()
+					else:
+						widget.container.start = ""
+						widget.container.end = ""
+		widget.oldtime = event.time
 
 	def mapclick(widget, event, data):
 		if (event.time - widget.oldtime) < 250:
@@ -496,7 +547,10 @@ def main():
 		dialog.destroy()
 		if (response == gtk.RESPONSE_OK) and (title.station_from != "") and (title.station_to != ""):
 			title.set_markup("<big>%s to %s</big>" % (Station[STATION_NAME][matchStation(title.station_from)], Station[STATION_NAME][matchStation(title.station_to)]))
-			route.get_buffer().set_text(guide(title.station_from, title.station_to))
+			plan = guide(title.station_from, title.station_to)
+			plan[0] = plan[0].replace('&', '&amp;')
+			route.plan = plan[1]
+			route.get_buffer().set_text(plan[0])
 			hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Routing complete")
 		hildon.hildon_gtk_window_set_progress_indicator(win, 0)
 
@@ -515,13 +569,19 @@ def main():
 	route = hildon.TextView()
 	route.set_editable(False)
 	route.set_wrap_mode(gtk.WRAP_WORD)
-	routebutton = gtk.Button("New route")
-	routebutton.connect("clicked", lambda button: showdialog(title, route))
+	route.plan = None
+	hbox = gtk.HBox()
+	routebutton_text = gtk.Button("Enter route (text)")
+	routebutton_text.connect("clicked", lambda button: showdialog(title, route))
+	routebutton_map = gtk.Button("Choose route (map)")
+	routebutton_map.connect("clicked", lambda button: togglemap(mapbuf, map_x, map_y, title, route, True))
+	hbox.pack_start(routebutton_text, True, True, 5)
+	hbox.pack_end(routebutton_map, True, True, 5)
 	mapbutton = gtk.Button("Tube map (double-click to return)")
-	mapbutton.connect("clicked", lambda button: togglemap(mapbuf, map_x, map_y))
+	mapbutton.connect("clicked", lambda button: togglemap(mapbuf, map_x, map_y, title, route))
 	vbox.pack_start(route, True, True, 5)
 	vbox.pack_start(gtk.Label(""), True, True, 5)
-	vbox.pack_start(routebutton, True, True, 5)
+	vbox.pack_start(hbox, True, True, 5)
 	vbox.pack_end(mapbutton, True, True, 5)
 	pannable_area = hildon.PannableArea()
 	pannable_area.add_with_viewport(vbox)
@@ -529,7 +589,6 @@ def main():
 	win.connect("destroy", gtk.main_quit, None)
 	win.show_all()
 
-	showdialog(title, route)
 	gtk.main()
 
 	###############################################################################
